@@ -34,6 +34,15 @@ function EventData(event, cal, start, end, description, status, location, organi
 	this.url = url;
 }
 
+function openTodoInfo(vtodo_array) {	
+	var windowW = 460;
+	var windowH = 275;
+	var url = "includes/todo.php?vtodo_array="+vtodo_array;
+	options = "scrollbars=yes,width="+windowW+",height="+windowH;
+	info = window.open(url, "Popup", options);
+	info.focus();
+}
+
 document.popup_data = new Array();
 //-->
 </script>
\ No newline at end of file
