@@ -7,11 +7,13 @@ include_once(BASE.'functions/draw_functions.php');
 include_once(BASE.'functions/overlapping_events.php');
 include_once(BASE.'functions/timezones.php');
 
+$php_started = getmicrotime();
+
 $fillTime = $day_start;
 $day_array = array ();
 while ($fillTime < $day_end) {
 	array_push ($day_array, $fillTime);
-	ereg ('([0-9]{2})([0-9]{2})', $fillTime, $dTime);
+	preg_match ('/([0-9]{2})([0-9]{2})/', $fillTime, $dTime);
 	$fill_h = $dTime[1];
 	$fill_min = $dTime[2];
 	$fill_min = sprintf('%02d', $fill_min + $gridLength);
@@ -24,27 +26,65 @@ while ($fillTime < $day_end) {
 
 // what date we want to get data for (for day calendar)
 if (!isset($getdate) || $getdate == '') $getdate = date('Ymd');
-ereg ('([0-9]{4})([0-9]{2})([0-9]{2})', $getdate, $day_array2);
+preg_match ("/([0-9]{4})([0-9]{2})([0-9]{2})/", $getdate, $day_array2);
 $this_day = $day_array2[3];
 $this_month = $day_array2[2];
 $this_year = $day_array2[1];
 
 // reading the file if it's allowed
 $parse_file = true;
-if (($is_webcal == false) && ($save_parsed_cals == 'yes') && ($cal != $ALL_CALENDARS_COMBINED)) {	
-	$realcal_mtime = filemtime($filename);
-	$parsedcal = $tmp_dir.'/parsedcal-'.$cal_filename.'-'.$this_year;
-	if (file_exists($parsedcal)) {
-		$parsedcal_mtime = filemtime($parsedcal);
-		if ($realcal_mtime == $parsedcal_mtime) {
+if ($save_parsed_cals == 'yes') {	
+	if (sizeof ($cal_filelist) > 1) {
+		$parsedcal = $tmp_dir.'/parsedcal-'.$cal_filename.'-'.$this_year;
+		if (file_exists($parsedcal)) {
 			$fd = fopen($parsedcal, 'r');
 			$contents = fread($fd, filesize($parsedcal));
 			fclose($fd);
 			$master_array = unserialize($contents);
-			if ($master_array['-1'] == 'valid cal file') {
-				$parse_file = false;
-				$calendar_name = $master_array['calendar_name'];
-				$calendar_tz = $master_array['calendar_tz'];
+			$z=1;
+			$y=0;
+			if (sizeof($master_array['-4']) == (sizeof($cal_filelist))) {
+				foreach ($master_array['-4'] as $temp_array) {
+					$mtime = $master_array['-4'][$z]['mtime'];
+					$fname = $master_array['-4'][$z]['filename'];
+					$wcalc = $master_array['-4'][$z]['webcal'];	
+					if ($wcalc == 'no') $realcal_mtime = filemtime($fname);
+					$webcal_mtime = strtotime("now -$webcal_hours hours");
+					if (($mtime == $realcal_mtime) && ($wcalc == 'no')) {
+						$y++;
+					} elseif (($wcalc == 'yes') && ($mtime > $webcal_mtime)) {
+						//echo date('H:i',$mtime). ' > '. date('H:i',$webcal_mtime);
+						$y++;
+					}
+					$z++;
+				}
+				if ($y == sizeof($cal_filelist)) {
+					if ($master_array['-1'] == 'valid cal file') {
+						$parse_file = false;
+						$calendar_name = $master_array['calendar_name'];
+						$calendar_tz = $master_array['calendar_tz'];
+					}
+				}
+			}
+		}
+		if ($parse_file == true) unset($master_array);
+	} else {
+		foreach ($cal_filelist as $filename) {
+			$realcal_mtime = filemtime($filename);
+			$parsedcal = $tmp_dir.'/parsedcal-'.$cal_filename.'-'.$this_year;
+			if (file_exists($parsedcal)) {
+				$parsedcal_mtime = filemtime($parsedcal);
+				if ($realcal_mtime == $parsedcal_mtime) {
+					$fd = fopen($parsedcal, 'r');
+					$contents = fread($fd, filesize($parsedcal));
+					fclose($fd);
+					$master_array = unserialize($contents);
+					if ($master_array['-1'] == 'valid cal file') {
+						$parse_file = false;
+						$calendar_name = $master_array['calendar_name'];
+						$calendar_tz = $master_array['calendar_tz'];
+					}
+				}
 			}
 		}
 	}
@@ -64,17 +104,32 @@ foreach ($cal_filelist as $filename) {
 	
 	if ($parse_file) {	
 		
-		$ifile = fopen($filename, "r");
-		if ($ifile == FALSE) exit(error($error_invalidcal_lang, $filename));
+		// Let's see if we're doing a webcal
+		$is_webcal == FALSE;
+		if (substr($filename, 0, 7) == 'http://' || substr($filename, 0, 8) == 'https://' || substr($filename, 0, 9) == 'webcal://') {
+			$is_webcal == TRUE;
+			$cal_webcalPrefix = str_replace('http://','webcal://',$filename);
+			$cal_httpPrefix = str_replace('webcal://','http://',$filename);
+			$cal_httpsPrefix = str_replace('webcal://','https://',$filename);
+			$cal_httpsPrefix = str_replace('http://','https://',$cal_httpsPrefix);
+			$filename = $cal_httpPrefix;
+			$master_array['-4'][$calnumber]['webcal'] = 'yes';
+			$actual_mtime = time();
+		} else {
+			$actual_mtime = @filemtime($filename);
+		}
+		
+		$ifile = @fopen($filename, "r");
+		if ($ifile == FALSE) exit(error($lang['l_error_cantopen'], $filename));
 		$nextline = fgets($ifile, 1024);
-		if (trim($nextline) != 'BEGIN:VCALENDAR') exit(error($error_invalidcal_lang, $filename));
+		if (trim($nextline) != 'BEGIN:VCALENDAR') exit(error($lang['l_error_invalidcal'], $filename));
 		
 		// Set a value so we can check to make sure $master_array contains valid data
 		$master_array['-1'] = 'valid cal file';
 	
 		// Set default calendar name - can be overridden by X-WR-CALNAME
 		$calendar_name = $cal_filename;
-		$master_array['calendar_name'] = $calendar_name;
+		$master_array['calendar_name'] 	= $calendar_name;
 		
 	// read file in line by line
 	// XXX end line is skipped because of the 1-line readahead
@@ -88,14 +143,19 @@ foreach ($cal_filelist as $filename) {
 				$nextline = ereg_replace("[\r\n]", "", $nextline);
 			}
 			$line = trim($line);
-			if ($line == 'BEGIN:VEVENT') {
+			
+		switch ($line) {
+			case 'BEGIN:VEVENT':
 				// each of these vars were being set to an empty string
 				unset (
 					$start_time, $end_time, $start_date, $end_date, $summary, 
 					$allday_start, $allday_end, $start, $end, $the_duration, 
 					$beginning, $rrule_array, $start_of_vevent, $description, $bd, $url, 
 					$valarm_description, $start_unixtime, $end_unixtime, $display_end_tmp, $end_time_tmp1, 
-					$recurrence_id, $uid, $class, $location, $rrule, $abs_until, $until_check
+					$recurrence_id, $uid, $class, $location, $rrule, $abs_until, $until_check,
+					$until, $bymonth, $byday, $bymonthday, $byweek, $byweekno, 
+					$byminute, $byhour, $bysecond, $byyearday, $bysetpos, $wkst,
+					$interval, $number
 				);
 					
 				$except_dates 	= array();
@@ -108,13 +168,12 @@ foreach ($cal_filelist as $filename) {
 				$attendee		= array();
 				$organizer		= array();
 				
-				unset(
-					$until, $bymonth, $byday, $bymonthday, $byweek, $byweekno, 
-					$byminute, $byhour, $bysecond, $byyearday, $bysetpos, $wkst,
-					$interval, $number
-				);
+				break;
+			
+			case 'END:VEVENT':
 				
-			} elseif ($line == 'END:VEVENT') {
+				if (!isset($url)) $url = '';
+				if (!isset($type)) $type = '';
 				
 				// Handle DURATION
 				if (!isset($end_unixtime) && isset($the_duration)) {
@@ -143,8 +202,10 @@ foreach ($cal_filelist as $filename) {
 				}
 				
 				if ($uid_valid && isset($processed[$uid]) && isset($recurrence_id['date'])) {
+					
 					$old_start_date = $processed[$uid][0];
 					$old_start_time = $processed[$uid][1];
+					if ($recurrence_id['value'] == 'DATE') $old_start_time = '-1';
 					$start_date_tmp = $recurrence_id['date'];
 					if (!isset($start_date)) $start_date = $old_start_date;
 					if (!isset($start_time)) $start_time = $master_array[$old_start_date][$old_start_time][$uid]['event_start'];
@@ -162,7 +223,11 @@ foreach ($cal_filelist as $filename) {
 					removeOverlap($start_date_tmp, $old_start_time, $uid);
 					if (isset($master_array[$start_date_tmp][$old_start_time][$uid])) {
 						unset($master_array[$start_date_tmp][$old_start_time][$uid]);  // SJBO added $uid twice here
+						if (sizeof($master_array[$start_date_tmp][$old_start_time]) == 0) {
+							unset($master_array[$start_date_tmp][$old_start_time]);
+						}
 					}
+					
 					$write_processed = false;
 				} else {
 					$write_processed = true;
@@ -184,7 +249,7 @@ foreach ($cal_filelist as $filename) {
 						$allday_end = ($start_date + 1);
 					}
 				}
-				if (isset($start_unixtime,$end_unixtime) && date('d',$start_unixtime) != date('d',$end_unixtime)) {
+				if (isset($start_unixtime,$end_unixtime) && date('Ymd',$start_unixtime) != date('Ymd',$end_unixtime)) {
 					$spans_day = true;
 					$bleed_check = (($start_unixtime - $end_unixtime) < (60*60*24)) ? '-1' : '0';
 				} else {
@@ -192,8 +257,8 @@ foreach ($cal_filelist as $filename) {
 					$bleed_check = 0;
 				}
 				if (isset($start_time) && $start_time != '') {
-					ereg ('([0-9]{2})([0-9]{2})', $start_time, $time);
-					ereg ('([0-9]{2})([0-9]{2})', $end_time, $time2);
+					preg_match ('/([0-9]{2})([0-9]{2})/', $start_time, $time);
+					preg_match ('/([0-9]{2})([0-9]{2})/', $end_time, $time2);
 					if (isset($start_unixtime) && isset($end_unixtime)) {
 						$length = $end_unixtime - $start_unixtime;
 					} else {
@@ -201,7 +266,7 @@ foreach ($cal_filelist as $filename) {
 					}
 					
 					$drawKey = drawEventTimes($start_time, $end_time);
-					ereg ('([0-9]{2})([0-9]{2})', $drawKey['draw_start'], $time3);
+					preg_match ('/([0-9]{2})([0-9]{2})/', $drawKey['draw_start'], $time3);
 					$hour = $time3[1];
 					$minute = $time3[2];
 				}
@@ -217,7 +282,7 @@ foreach ($cal_filelist as $filename) {
 				if ($uid_valid && $write_processed) {
 					if (!isset($hour)) $hour = 00;
 					if (!isset($minute)) $minute = 00;
-					$processed[$uid] = array($start_date,($hour.$minute));
+					$processed[$uid] = array($start_date,($hour.$minute), $type);
 				}
 							
 				// Handling of the all day events
@@ -232,7 +297,7 @@ foreach ($cal_filelist as $filename) {
 						$end = strtotime('+1 day', $start);
 					}
 					// Changed for 1.0, basically write out the entire event if it starts while the array is written.
-					if ($start < $mArray_end) {
+					if (($start < $mArray_end) && ($start < $end)) {
 						while (($start != $end) && ($start < $mArray_end)) {
 							$start_date2 = date('Ymd', $start);
 							$master_array[($start_date2)][('-1')][$uid]= array ('event_text' => $summary, 'description' => $description, 'location' => $location, 'organizer' => serialize($organizer), 'attendee' => serialize($attendee), 'calnumber' => $calnumber, 'calname' => $actual_calname, 'url' => $url, 'status' => $status, 'class' => $class );
@@ -310,8 +375,8 @@ foreach ($cal_filelist as $filename) {
 					$start_date_time = strtotime($start_date);
 					$this_month_start_time = strtotime($this_year.$this_month.'01');
 					if ($save_parsed_cals == 'yes' && !$is_webcal) {
-						$start_range_time = strtotime($this_year.'-01-01 -1 month -2 days');
-						$end_range_time = strtotime($this_year.'-12-31 +1 month +2 days');
+						$start_range_time = strtotime($this_year.'-01-01 -2 weeks');
+						$end_range_time = strtotime($this_year.'-12-31 +2 weeks');
 					} else {
 						$start_range_time = strtotime('-1 month -2 day', $this_month_start_time);
 						$end_range_time = strtotime('+2 month +2 day', $this_month_start_time);
@@ -336,8 +401,8 @@ foreach ($cal_filelist as $filename) {
 								$master_array[($start_date)][($hour.$minute)][$uid]['recur'][$key] = $count;
 								break;
 							case 'UNTIL':
-								$until = ereg_replace('T', '', $val);
-								$until = ereg_replace('Z', '', $until);
+								$until = str_replace('T', '', $val);
+								$until = str_replace('Z', '', $until);
 								if (strlen($until) == 8) $until = $until.'235959';
 								$abs_until = $until;
 								ereg ('([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})', $until, $regs);
@@ -464,8 +529,9 @@ foreach ($cal_filelist as $filename) {
 													}
 													break;
 												case 'MONTHLY':
-													if (!isset($bymonth)) $bymonth = array(1,2,3,4,5,6,7,8,9,10,11,12);
+													if (empty($bymonth)) $bymonth = array(1,2,3,4,5,6,7,8,9,10,11,12);
 													$next_range_time = strtotime(date('Y-m-01', $next_range_time));
+													$next_date_time = $next_date_time;
 													if ((isset($bymonthday)) && (!isset($byday))) {
 														foreach($bymonthday as $day) {
 															$year = date('Y', $next_range_time);
@@ -475,10 +541,15 @@ foreach ($cal_filelist as $filename) {
 																$recur_data[] = $next_date_time;
 															}
 														}
-													} else {
+													} elseif (is_array($byday)) {
 														foreach($byday as $day) {
 															ereg ('([-\+]{0,1})?([0-9]{1})?([A-Z]{2})', $day, $byday_arr);
-															$nth = $byday_arr[2]-1;
+															//Added for 2.0 when no modifier is set
+															if ($byday_arr[2] != '') {
+																$nth = $byday_arr[2]-1;
+															} else {
+																$nth = 0;
+															}
 															$on_day = two2threeCharDays($byday_arr[3]);
 															$on_day_num = two2threeCharDays($byday_arr[3],false);
 															if ((isset($byday_arr[1])) && ($byday_arr[1] == '-')) {
@@ -490,7 +561,7 @@ foreach ($cal_filelist as $filename) {
 																if (in_array($month, $bymonth)) {
 																	$recur_data[] = $next_date_time;
 																}
-															} elseif (isset($bymonthday)) {
+															} elseif (isset($bymonthday) && (!empty($bymonthday))) {
 																// This supports MONTHLY where BYDAY and BYMONTH are both set
 																foreach($bymonthday as $day) {
 																	$year 	= date('Y', $next_range_time);
@@ -511,24 +582,31 @@ foreach ($cal_filelist as $filename) {
 																}
 															}
 															$next_date = date('Ymd', $next_date_time);
-															//$recur_data[] = $next_date_time;
 														}
 													}
 													break;
 												case 'YEARLY':
-												
-													if (!isset($bymonth)) {
+													if ((!isset($bymonth)) || (sizeof($bymonth) == 0)) {
 														$m = date('m', $start_date_time);
 														$bymonth = array("$m");
 													}	
 													foreach($bymonth as $month) {
+														// FIXME: Jan 1 does not display in December
 														$year = date('Y', $next_range_time);
+														if (($year != $this_year) && ($month < 3)) {
+															$year = $this_year;
+														}
 														if ((isset($byday)) && (is_array($byday))) {
 															$checkdate_time = mktime(0,0,0,$month,1,$year);
 															foreach($byday as $day) {
 																ereg ('([-\+]{0,1})?([0-9]{1})?([A-Z]{2})', $day, $byday_arr);
-																$nth = $byday_arr[2]-1;
+																if ($byday_arr[2] != '') {
+																	$nth = $byday_arr[2]-1;
+																} else {
+																	$nth = 0;
+																}
 																$on_day = two2threeCharDays($byday_arr[3]);
+																$on_day_num = two2threeCharDays($byday_arr[3],false);
 																if ($byday_arr[1] == '-') {
 																	$last_day_tmp = date('t',$checkdate_time);
 																	$checkdate_time = strtotime(date('Y-m-'.$last_day_tmp, $checkdate_time));
@@ -541,6 +619,7 @@ foreach ($cal_filelist as $filename) {
 														} else {
 															$day 	= date('d', $start_date_time);
 															$next_date_time = mktime(0,0,0,$month,$day,$year);
+															//echo date('Ymd',$next_date_time).$summary.'<br>';
 														}
 														$recur_data[] = $next_date_time;
 													}
@@ -558,7 +637,7 @@ foreach ($cal_filelist as $filename) {
 															}
 															$recur_data[] = $next_date_time;
 														}
-													}
+													} 
 													break;
 												default:
 													// anything else we need to end the loop
@@ -667,9 +746,8 @@ foreach ($cal_filelist as $filename) {
 			   // Clear event data now that it's been saved.
 			   unset($start_time, $start_time_tmp, $end_time, $end_time_tmp, $start_unixtime, $start_unixtime_tmp, $end_unixtime, $end_unixtime_tmp, $summary, $length, $nbrOfOverlaps, $description, $status, $class, $location, $organizer, $attendee);
 
-
-			// Begin VTODO Support
-			} elseif ($line == 'END:VTODO') {
+				break;
+			case 'END:VTODO':
 				if ((!$vtodo_priority) && ($status == 'COMPLETED')) {
 					$vtodo_sort = 11;
 				} elseif (!$vtodo_priority) { 
@@ -680,13 +758,20 @@ foreach ($cal_filelist as $filename) {
 				$master_array['-2']["$vtodo_sort"]["$uid"] = array ('start_date' => $start_date, 'start_time' => $start_time, 'vtodo_text' => $summary, 'due_date'=> $due_date, 'due_time'=> $due_time, 'completed_date' => $completed_date, 'completed_time' => $completed_time, 'priority' => $vtodo_priority, 'status' => $status, 'class' => $class, 'categories' => $vtodo_categories, 'description' => $description, 'calname' => $actual_calname);
 				unset ($start_date, $start_time, $due_date, $due_time, $completed_date, $completed_time, $vtodo_priority, $status, $class, $vtodo_categories, $summary, $description);
 				$vtodo_set = FALSE;
-			} elseif ($line == 'BEGIN:VTODO') {
+				
+				break;
+				
+			case 'BEGIN:VTODO':
 				$vtodo_set = TRUE;
-			} elseif ($line == 'BEGIN:VALARM') {
+				break;
+			case 'BEGIN:VALARM':
 				$valarm_set = TRUE;
-			} elseif ($line == 'END:VALARM') {
+				break;
+			case 'END:VALARM':
 				$valarm_set = FALSE;
-			} else {
+				break;
+				
+			default:
 		
 				unset ($field, $data, $prop_pos, $property);
 				ereg ("([^:]+):(.*)", $line, $line);
@@ -706,8 +791,8 @@ foreach ($cal_filelist as $filename) {
 						$data = str_replace ('/softwarestudio.org/Olson_20011030_5/', '', $data);
 						$zulu_time = false;
 						if (substr($data,-1) == 'Z') $zulu_time = true;
-						$data = ereg_replace('T', '', $data);
-						$data = ereg_replace('Z', '', $data);
+						$data = str_replace('T', '', $data);
+						$data = str_replace('Z', '', $data);
 						if (preg_match("/^DUE;VALUE=DATE/i", $field))  {
 							$allday_start = $data;
 							$start_date = $allday_start;
@@ -723,8 +808,8 @@ foreach ($cal_filelist as $filename) {
 							}
 			
 							ereg ('([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})', $data, $regs);
-							$start_date = $regs[1] . $regs[2] . $regs[3];
-							$start_time = $regs[4] . $regs[5];
+							$due_date = $regs[1] . $regs[2] . $regs[3];
+							$due_time = $regs[4] . $regs[5];
 							$start_unixtime = mktime($regs[4], $regs[5], 0, $regs[2], $regs[3], $regs[1]);
 			
 							$dlst = date('I', $start_unixtime);
@@ -755,8 +840,8 @@ foreach ($cal_filelist as $filename) {
 						$data = str_replace ('/softwarestudio.org/Olson_20011030_5/', '', $data);
 						$zulu_time = false;
 						if (substr($data,-1) == 'Z') $zulu_time = true;
-						$data = ereg_replace('T', '', $data);
-						$data = ereg_replace('Z', '', $data);
+						$data = str_replace('T', '', $data);
+						$data = str_replace('Z', '', $data);
 						if (preg_match("/^COMPLETED;VALUE=DATE/i", $field))  {
 							$allday_start = $data;
 							$start_date = $allday_start;
@@ -770,8 +855,8 @@ foreach ($cal_filelist as $filename) {
 							}
 			
 							ereg ('([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})', $data, $regs);
-							$start_date = $regs[1] . $regs[2] . $regs[3];
-							$start_time = $regs[4] . $regs[5];
+							$completed_date = $regs[1] . $regs[2] . $regs[3];
+							$completed_time = $regs[4] . $regs[5];
 							$start_unixtime = mktime($regs[4], $regs[5], 0, $regs[2], $regs[3], $regs[1]);
 			
 							$dlst = date('I', $start_unixtime);
@@ -817,17 +902,16 @@ foreach ($cal_filelist as $filename) {
 					// End VTODO Parsing				
 						
 					case 'DTSTART':
-						$field = str_replace ('/softwarestudio.org/Olson_20011030_5/', '', $field);
+						$data = str_replace ('/softwarestudio.org/Olson_20011030_5/', '', $data);
 						$zulu_time = false;
 						if (substr($data,-1) == 'Z') $zulu_time = true;
-						$data = ereg_replace('T', '', $data);
-						$data = ereg_replace('Z', '', $data);
-						$field = ereg_replace(';VALUE=DATE-TIME', '', $field); 
+						$data = str_replace('T', '', $data);
+						$data = str_replace('Z', '', $data);
+						$field = str_replace(';VALUE=DATE-TIME', '', $field); 
 						if ((preg_match("/^DTSTART;VALUE=DATE/i", $field)) || (ereg ('^([0-9]{4})([0-9]{2})([0-9]{2})$', $data)))  {
 							ereg ('([0-9]{4})([0-9]{2})([0-9]{2})', $data, $dtstart_check);
-							if ($dtstart_check[1] < 1969) { 
-								$dtstart_check[1] = '1990';
-								$data = $dtstart_check[1].$dtstart_check[2].$dtstart_check[3];
+							if ($dtstart_check[1] < 1970) { 
+								$data = '1971'.$dtstart_check[2].$dtstart_check[3];
 							}
 							$allday_start = $data;
 							$start_date = $allday_start;
@@ -841,7 +925,7 @@ foreach ($cal_filelist as $filename) {
 								$tz_dtstart = 'GMT';
 							}
 			
-							ereg ('([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})', $data, $regs);
+							preg_match ('/([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})/', $data, $regs);
 							$start_date = $regs[1] . $regs[2] . $regs[3];
 							$start_time = $regs[4] . $regs[5];
 							$start_unixtime = mktime($regs[4], $regs[5], 0, $regs[2], $regs[3], $regs[1]);
@@ -870,18 +954,17 @@ foreach ($cal_filelist as $filename) {
 						}
 						break;
 						
-					case 'DTEND';
+					case 'DTEND':
 						$data = str_replace ('/softwarestudio.org/Olson_20011030_5/', '', $data);
 						$zulu_time = false;
 						if (substr($data,-1) == 'Z') $zulu_time = true;
-						$data = ereg_replace('T', '', $data);
-						$data = ereg_replace('Z', '', $data);
-						$field = ereg_replace(';VALUE=DATE-TIME', '', $field); 
+						$data = str_replace('T', '', $data);
+						$data = str_replace('Z', '', $data);
+						$field = str_replace(';VALUE=DATE-TIME', '', $field); 
 						if (preg_match("/^DTEND;VALUE=DATE/i", $field))  {
-							ereg ('([0-9]{4})([0-9]{2})([0-9]{2})', $data, $dtend_check);
-							if ($dtend_check[1] < 1969) { 
-								$dtend_check[1] = '1990';
-								$data = $dtend_check[1].$dtend_check[2].$dtend_check[3];
+							preg_match ('/([0-9]{4})([0-9]{2})([0-9]{2})/', $data, $dtend_check);
+							if ($dtend_check[1] < 1970) { 
+								$data = '1971'.$dtend_check[2].$dtend_check[3];
 							}
 							$allday_end = $data;
 						} else {
@@ -893,7 +976,7 @@ foreach ($cal_filelist as $filename) {
 								$tz_dtend = 'GMT';
 							}
 							
-							ereg ('([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})', $data, $regs);
+							preg_match ('/([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})/', $data, $regs);
 							$end_date = $regs[1] . $regs[2] . $regs[3];
 							$end_time = $regs[4] . $regs[5];
 							$end_unixtime = mktime($regs[4], $regs[5], 0, $regs[2], $regs[3], $regs[1]);
@@ -918,17 +1001,23 @@ foreach ($cal_filelist as $filename) {
 					case 'EXDATE':
 						$data = split(",", $data);
 						foreach ($data as $exdata) {
-							$exdata = ereg_replace('T', '', $exdata);
-							$exdata = ereg_replace('Z', '', $exdata);
-							ereg ('([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})', $exdata, $regs);
+							$exdata = str_replace('T', '', $exdata);
+							$exdata = str_replace('Z', '', $exdata);
+							preg_match ('/([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})/', $exdata, $regs);
 							$except_dates[] = $regs[1] . $regs[2] . $regs[3];
-							$except_times[] = $regs[4] . $regs[5];
+							// Added for Evolution, since they dont think they need to tell me which time to exclude.
+							if (($regs[4] == '') && ($start_time != '')) { 
+								$except_times[] = $start_time;
+							} else {
+								$except_times[] = $regs[4] . $regs[5];
+							}
 						}
 						break;
 						
 					case 'SUMMARY':
-						$data = str_replace("\\n", "<br>", $data);
-						$data = str_replace("\\r", "<br>", $data);
+						$data = str_replace("\\n", "<br />", $data);
+						$data = str_replace("\\r", "<br />", $data);
+						$data = str_replace('$', '&#36;', $data);
 						$data = htmlentities(urlencode($data));
 						if ($valarm_set == FALSE) { 
 							$summary = $data;
@@ -938,8 +1027,9 @@ foreach ($cal_filelist as $filename) {
 						break;
 						
 					case 'DESCRIPTION':
-						$data = str_replace("\\n", "<br>", $data);
-						$data = str_replace("\\r", "<br>", $data);
+						$data = str_replace("\\n", "<br />", $data);
+						$data = str_replace("\\r", "<br />", $data);
+						$data = str_replace('$', '&#36;', $data);
 						$data = htmlentities(urlencode($data));
 						if ($valarm_set == FALSE) { 
 							$description = $data;
@@ -966,8 +1056,8 @@ foreach ($cal_filelist as $filename) {
 						}
 						unset($parts, $part, $eachval);
 						
-						$data = ereg_replace('T', '', $data);
-						$data = ereg_replace('Z', '', $data);
+						$data = str_replace('T', '', $data);
+						$data = str_replace('Z', '', $data);
 						ereg ('([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{0,2})([0-9]{0,2})', $data, $regs);
 						$recurrence_id['date'] = $regs[1] . $regs[2] . $regs[3];
 						$recurrence_id['time'] = $regs[4] . $regs[5];
@@ -1006,17 +1096,17 @@ foreach ($cal_filelist as $filename) {
 					case 'DURATION':
 						if (($first_duration == TRUE) && (!stristr($field, '=DURATION'))) {
 							ereg ('^P([0-9]{1,2}[W])?([0-9]{1,2}[D])?([T]{0,1})?([0-9]{1,2}[H])?([0-9]{1,2}[M])?([0-9]{1,2}[S])?', $data, $duration); 
-							$weeks 			= ereg_replace('W', '', $duration[1]); 
-							$days 			= ereg_replace('D', '', $duration[2]); 
-							$hours 			= ereg_replace('H', '', $duration[4]); 
-							$minutes 		= ereg_replace('M', '', $duration[5]); 
-							$seconds 		= ereg_replace('S', '', $duration[6]); 
+							$weeks 			= str_replace('W', '', $duration[1]); 
+							$days 			= str_replace('D', '', $duration[2]); 
+							$hours 			= str_replace('H', '', $duration[4]); 
+							$minutes 		= str_replace('M', '', $duration[5]); 
+							$seconds 		= str_replace('S', '', $duration[6]); 
 							$the_duration 	= ($weeks * 60 * 60 * 24 * 7) + ($days * 60 * 60 * 24) + ($hours * 60 * 60) + ($minutes * 60) + ($seconds);
 							$first_duration = FALSE;
 						}	
 						break;
 					case 'RRULE':
-						$data = ereg_replace ('RRULE:', '', $data);
+						$data = str_replace ('RRULE:', '', $data);
 						$rrule = split (';', $data);
 						foreach ($rrule as $recur) {
 							ereg ('(.*)=(.*)', $recur, $regs);
@@ -1024,18 +1114,18 @@ foreach ($cal_filelist as $filename) {
 						}
 						break;
 					case 'ATTENDEE':
-						$field 		= ereg_replace("ATTENDEE;CN=", "", $field);
-						$data 		= ereg_replace ("mailto:", "", $data);
+						$field 		= str_replace("ATTENDEE;CN=", "", $field);
+						$data 		= str_replace ("mailto:", "", $data);
 						$attendee[] = array ('name' => $field, 'email' => $data);
 						break;
 					case 'ORGANIZER':
-						$field 		 = ereg_replace("ORGANIZER;CN=", "", $field);
-						$data 		 = ereg_replace ("mailto:", "", $data);
+						$field 		 = str_replace("ORGANIZER;CN=", "", $field);
+						$data 		 = str_replace ("mailto:", "", $data);
 						$organizer[] = array ('name' => $field, 'email' => $data);
 						break;
 					case 'LOCATION':
-						$data = str_replace("\\n", "<br>", $data);
-						$data = str_replace("\\r", "<br>", $data);
+						$data = str_replace("\\n", "<br />", $data);
+						$data = str_replace("\\r", "<br />", $data);
 						$location = $data;
 						break;
 					case 'URL':
@@ -1045,10 +1135,13 @@ foreach ($cal_filelist as $filename) {
 			}
 		}
 	}
+	if (!isset($master_array['-3'][$calnumber])) $master_array['-3'][$calnumber] = $actual_calname;
+	if (!isset($master_array['-4'][$calnumber]['mtime'])) $master_array['-4'][$calnumber]['mtime'] = $actual_mtime;
+	if (!isset($master_array['-4'][$calnumber]['filename'])) $master_array['-4'][$calnumber]['filename'] = $filename;
+	if (!isset($master_array['-4'][$calnumber]['webcal'])) $master_array['-4'][$calnumber]['webcal'] = 'no';
 	$calnumber = $calnumber + 1;
 }
 
-
 if ($parse_file) {	
 	// Sort the array by absolute date.
 	if (isset($master_array) && is_array($master_array)) { 
@@ -1065,17 +1158,16 @@ if ($parse_file) {
 	}
 	
 	// write the new master array to the file
-	if (isset($master_array) && is_array($master_array) && $save_parsed_cals == 'yes' && $is_webcal == FALSE && $cal != $ALL_CALENDARS_COMBINED) {
+	if (isset($master_array) && is_array($master_array) && $save_parsed_cals == 'yes') {
 		$write_me = serialize($master_array);
-		$fd = fopen($parsedcal, 'w');
-		fwrite($fd, $write_me);
-		fclose($fd);
-		touch($parsedcal, $realcal_mtime);
+		$fd = @fopen($parsedcal, 'w');
+		if ($fd == FALSE) exit(error($lang['l_error_cache'], $filename));
+		@fwrite($fd, $write_me);
+		@fclose($fd);
+		@touch($parsedcal, $realcal_mtime);
 	}
 }
 
-
-
 //If you want to see the values in the arrays, uncomment below.
 
 //print '<pre>';
@@ -1090,5 +1182,7 @@ if ($parse_file) {
 if ($cal == $ALL_CALENDARS_COMBINED) {
 	$calendar_name = $all_cal_comb_lang;
 }
+
+$template_started = getmicrotime();
 					
 ?>
