@@ -2,8 +2,8 @@
 
 if (!isset($ALL_CALENDARS_COMBINED))  $ALL_CALENDARS_COMBINED = 'all_calendars_combined971';
 include "./config.inc.php";
-if (isset($HTTP_COOKIE_VARS['phpicalendar'])) {
-	$phpicalendar 		= unserialize(stripslashes($HTTP_COOKIE_VARS['phpicalendar']));
+if (isset($_COOKIE['phpicalendar'])) {
+	$phpicalendar 		= unserialize(stripslashes($_COOKIE['phpicalendar']));
 	$default_view 		= $phpicalendar['cookie_view'];
 }
 if ($printview_default == 'yes') {
@@ -11,6 +11,8 @@ if ($printview_default == 'yes') {
 } else {
 	$default_view = "$default_view" . ".php";
 }
-header("Location: $default_view");
+/*header("Location: $default_view");*/
+
+include( $default_view );
 
 ?>
\ No newline at end of file
